﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;

namespace UniversityDatabaseImplement.Implements
{
    internal class ExaminationResultStorage : IExaminationResultStorage
    {
        public List<ExaminationResultViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.ExaminationResults.Select(x => x.GetViewModel).ToList();
        }
        public List<ExaminationResultViewModel> GetFilteredList(ExaminationResultSearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.ExaminationResults
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id)
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ExaminationResultViewModel? GetElement(ExaminationResultSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.ExaminationResults
            .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public ExaminationResultViewModel? Insert(ExaminationResultBindingModel model)
        {
            var newExaminationResult = ExaminationResult.Create(model);
            if (newExaminationResult == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.ExaminationResults.Add(newExaminationResult);
            context.SaveChanges();
            return newExaminationResult.GetViewModel;
        }

        public ExaminationResultViewModel? Update(ExaminationResultBindingModel model)
        {
            using var context = new UniversityDatabase();
            var examinationResult = context.ExaminationResults.FirstOrDefault(x => x.Id == model.Id);
            if (examinationResult == null)
            {
                return null;
            }
            examinationResult.Update(model);
            context.SaveChanges();
            return examinationResult.GetViewModel;
        }
        public ExaminationResultViewModel? Delete(ExaminationResultBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.ExaminationResults.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.ExaminationResults.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
